@@ -781,6 +781,13 @@ labeled_image_get_image_bounds_fill (const EelLabeledImage *labeled_image)
 			image_bounds.x1 = bounds.x1;
 			image_bounds.y1 = content_bounds.y0 + image_dimensions.height;
 			break;
+
+		default:
+			image_bounds.x0 = 0;
+			image_bounds.y0 = 0;
+			image_bounds.x1 = 0;
+			image_bounds.y1 = 0;
+			g_assert_not_reached ();
 		}
 	}
 
@@ -864,6 +871,11 @@ eel_labeled_image_get_image_bounds (const EelLabeledImage *labeled_image)
 			}	
 
 			break;
+
+		default:
+			image_bounds.x0 = 0;
+			image_bounds.y0 = 0;
+			g_assert_not_reached ();
 		}
 	}
 	
@@ -925,6 +937,13 @@ labeled_image_get_label_bounds_fill (const EelLabeledImage *labeled_image)
 			label_bounds.x1 = bounds.x1;
 			label_bounds.y1 = bounds.y1;
 			break;
+
+		default:
+			label_bounds.x0 = 0;
+			label_bounds.y0 = 0;
+			label_bounds.x1 = 0;
+			label_bounds.y1 = 0;
+			g_assert_not_reached ();
 		}
 	}
 
@@ -990,6 +1009,11 @@ eel_labeled_image_get_label_bounds (const EelLabeledImage *labeled_image)
 				(eel_art_irect_get_width (content_bounds) - label_dimensions.width) / 2;
 			label_bounds.y0 = content_bounds.y1 - label_dimensions.height;
 			break;
+
+		default:
+			label_bounds.x0 = 0;
+			label_bounds.y0 = 0;
+			g_assert_not_reached ();
 		}
 	}
 	
